#ifndef __CMP_H__
#define __CMP_H__
#include <stdint.h>

#include "common.h"
#include "sci_common.h"

typedef enum
{
	CMP_Channel_0  = 1,
	CMP_Channel_1  = 2,
}	CMP_Channel_t;

typedef enum
{
	CMPx_PositiveSide_VCIN0 = 0,   // external VCIN0 PIN
	CMPx_PositiveSide_VCIN1 = 1,   // external VCIN1 PIN
	CMPx_PositiveSide_VCIN2 = 2,   // external VCIN2 PIN
	CMPx_PositiveSide_VCIN3 = 3,   // external VCIN3 PIN
	CMPx_PositiveSide_VCIN4 = 4,   // external VCIN4 PIN
	CMPx_PositiveSide_VCIN5 = 5,   // external VCIN5 PIN
	CMP0_PositiveSide_OPAO  = 5,   // OPAOUT signal
    
/*******define negative****************************************/
	CMPx_NegativeSide_VCIN0     = 0,	//cmp negative can choose external VCIN0 PIN
	CMPx_NegativeSide_VCIN1     = 1,	//cmp negative can choose external VCIN1 PIN
	CMPx_NegativeSide_VCIN2     = 2,	//cmp negative can choose external VCIN2 PIN
	CMPx_NegativeSide_DAC       = 2,  // 8-bit DAC output 
	CMPx_NegativeSide_BGR       = 3,  //internal voltage reference 1.45v
	
}CMP_Side_t;

typedef enum
{
	 CMP_HyMode_None    = 0,  // None Hy
	 CMP_HyMode_Postive = 1,  //positive hy
	 CMP_HyMode_Negative =2,  //negative hy
	 CMP_HyMode_Both    = 3,  //both side hy	 
}CMP_HyMode_t;

typedef enum
{
	 CMP_HyVal_None   = 0,  // None Hy
	 CMP_HyVal_20mv   = 1,  //Comparator Hy 20mV
	 CMP_HyVal_40mv   = 2,  //Comparator Hy 40mV
	 CMP_HyVal_60mv   = 3,  //Comparator Hy 60mV
}CMP_HyVal_t;

typedef enum
{
	 CMP_IntReq_Rising   = 0,  // Interrupt request at comparator rising edge
	 CMP_IntReq_Falling  = 1, //Interrupt request at comparator falling edge
	 CMP_IntReq_Both     = 2, //Interrupt request at comparator both edge
}CMP_IntReq_t;

typedef enum
{
	 CMP_Int_Disable = 0, // Interrupt disable
	 CMP_Int_Enable  = 1, //Interrupt enable
}CMP_Int_t;
typedef enum
{
	 CMP_Filter_None   = 0, // None Filter
	 CMP_Filter_Fclk  = 1,  // Comparator filter enabled, sampling at fCLK
	 CMP_Filter_Fclk8 = 2,  // Comparator filter enabled, sampling at fCLK/8 
	 CMP_Filter_Fclk32= 3,  // Comparator filter enabled, sampling at fCLK/32
}CMP_Filter_t;

typedef enum
{
	 CMP_Out_Disable = 0, // Disable cmp result to outport
	 CMP_Out_Enable  = 1, //Enable cmp result to outport
}CMP_Out_t;
typedef enum
{
	 CMP_Out_Noraml   = 0, 
	 CMP_Out_Reverse  = 1, 
}CMP_Polarity_t;
typedef struct
{
	CMP_Channel_t CMP_Channel;	/*!< Specifies the channel of comparator.*/
										   	
	CMP_Side_t CMP_PositiveSide;/*!< Specifies the input signal of positive side of comparator 
								 This parameter can be a value of @ref CMP_Side_t */
	CMP_Side_t CMP_NegativeSide;/*!< Specifies the input signal of negative side of comparator 
								 This parameter can be a value of @ref CMP_Side_t */
//	CMP_Vref_t CMP_NegVref;  /*!< Specifies the comparator use vdd or avrefp when negative side is CMP0_NegativeSide_IVREF
//								 This parameter can be a value of @ref CMP_Vref_t */
//	uint8_t CMP_NegVrefVol;  /*!< Specifies the negative side voltage */
	
	CMP_HyMode_t CMP_HyMode; /*!< Specifies the comparator hy mode
								 This parameter can be a value of @ref CMP_HyMode_t */
								 	
	CMP_HyVal_t CMP_HyVal; /*!< Specifies the comparator hy voltage
								 This parameter can be a value of @ref CMP_HyVal_t */	
								 	
	CMP_Filter_t CMP_Filter; /*!< Specifies the comparator hy voltage
							  This parameter can be a value of @ref CMP_HyVal_t */	 
							  	
	CMP_IntReq_t CMP_IntReq;/*!< Specifies generate comparator interrupt request condition 
							   This parameter can be a value of @ref CMP_IntReq_t */
							   	
	CMP_Int_t CMP_Int;/*!< Specifies generate comparator interrupt diasble or enbale
								This parameter can be a value of @ref CMP_Int_t */
	CMP_Out_t CMP_Out2port; /*!< Specifies the comparator result whether output to port or not
								 This parameter can be a value of @ref CMP_Out_t */
	CMP_Polarity_t CMP_Polarity; /*!< Specifies the comparator result whether output to port or not
								 This parameter can be a value of @ref CMP_Out_t */									 
}CMP_InitTypeDef;

void CMP_Init(CMP_InitTypeDef* CMP_InitStruct);
void CMP_Start(CMP_Channel_t ch);
void CMP_Stop(CMP_Channel_t ch);
void CMP_Input_Select(CMP_Channel_t ch, CMP_Side_t positive_side, CMP_Side_t negative_side);
void CMP_INTConfig(CMP_Channel_t ch);
void CMP_INT_Stop(CMP_Channel_t ch);

#endif

