/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    lcd.c
* @brief   This file implements device driver for LCD module.
* @version 1.0.0
* @date    2022/8/24
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "CMS32H6157.h"
#include "userdefine.h"
#include "lcd.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Macro definitions 
***********************************************************************************************************************/
#define IN_RES      (0x00U) /* Internal resistance division method */
#define IN_BOOST    (0x40U) /* internal voltage boosting method */
#define CAP_SPLT    (0x80U) /* capacitor split method */
#define EXT_RES     (0xC0U) /* external resistance division method */

#define LCD_MODE_SEL (EXT_RES)



/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: LCD_Init
* Description  : This function initializes the LCD module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LCDApi_Init(LCD_InitTypeDef* LCD_InitStruct)
{
	uint8_t tmpreg = 0x00;
	assert_param(IS_LCD_Bias_Select(LCD_InitStruct->Bias));
	assert_param(IS_LCD_CLK_Select(LCD_InitStruct->CLKSel));
	assert_param(IS_LCD_Wave_Select(LCD_InitStruct->LCDWave));
	assert_param(IS_LCD_BaseVoltage_Select(LCD_InitStruct->VoltageVM));
	assert_param(IS_LCD_TimeSlice_Select(LCD_InitStruct->TimeSlice));
	assert_param(IS_LCD_Voltage_Mode(LCD_InitStruct->VoltageMode));
	assert_param(IS_LCD_Voltage_Select(LCD_InitStruct->VContrast));
	
	/* LCD voltage split method, wave select, duty select and bias select for register operate */
	tmpreg = LCD->LCDM0;
	tmpreg &= ~(LCD_LCDM0_MDSET_Msk | LCD_LCDM0_LWAVE_Msk | LCD_LCDM0_LDTY_Msk
							| LCD_LCDM0_LBAS_Msk);
	tmpreg = ((LCD_InitStruct->VoltageMode ) << LCD_LCDM0_MDSET_Pos)
					| ((LCD_InitStruct->Bias) << LCD_LCDM0_LBAS_Pos) | ((LCD_InitStruct->LCDWave)<<LCD_LCDM0_LWAVE_Pos)
	            | ((LCD_InitStruct->TimeSlice)<<LCD_LCDM0_LDTY_Pos);
	LCD->LCDM0 = tmpreg;
	
	/* LCD rising voltage initial voltage set for register operate */
	tmpreg = LCD->LCDM1;
	tmpreg &= ~LCD_LCDM1_LCDVLM_Msk;
	tmpreg |= (LCD_InitStruct->VoltageVM) << LCD_LCDM1_LCDVLM_Pos;
	LCD->LCDM1 = tmpreg;
	
	/* LCD clock set for register operate */
	LCD->LCDC0 = LCD_InitStruct->CLKSel;
	
	/* LCD contrast voltage set for register operate */
	LCD->VLCD = LCD_InitStruct->VContrast;
}
void LCDApi0_Init(void)
{
	
//	assert_param(IS_UART_ALL_PERIPH(UARTx));
	
    LCD->LCDM1 &= ~LCD_LCDM1_LCDON_Msk;    /* disable LCD clock operation */
    LCD->LCDM0 = _00_LCD_DISPLAY_WAVEFORM_A | _0D_LCD_DISPLAY_MODE1;
    LCD->LCDM0 |= _80_LCD_VOLTAGE_MODE_CAPACITOR;
    /* Set CAPL and CAPH pins */
    PPORGA->ISCLCD &= (uint8_t)~_01_LCD_CAPLH_BUFFER_VALID;
#if (LCD_MODE_SEL==IN_RES)
#elif (LCD_MODE_SEL==IN_BOOST)
#elif (LCD_MODE_SEL==CAP_SPLT)
#elif (LCD_MODE_SEL==EXT_RES)
#endif

    PORT->PMCA=0x0000;
    PORT->PMCB=0x0000;
    PORT->PMCC=0x0000;
    PORT->PMCD=0x0000;

    PORT->PA=0x00000;
    PORT->PB=0x00000;
    PORT->PC=0x00000;
    PORT->PD=0x00000;
    PORT->PH=0x00000;

    PPORGA->SEGEN0 = 0x00ff;
    PPORGA->SEGEN1 = 0x0fff;
    PPORGA->SEGEN2 = 0xffff;
    PPORGA->SEGEN3 = 0x03ff;

    LCD->LCDM1 |= _00_LCD_DISPLAY_PATTERN_A;
    LCD->LCDC0 = _06_LCD_CLOCK_FSUB_FIL_7;
		LCD_Voltage_On();
		LCD_Start();
}

/***********************************************************************************************************************
* Function Name: LCD_VoltageMode
* Description  : This function select voltage generate method.
* Arguments    : AreaAB
*                  LCD_Voltage_InterRES: internal ressitance
*									 LCD_Voltage_InterRise: internal rising voltage
*									 LCD_Voltage_CapDiv: capacitance divid
*									 LCD_Voltage_ExterRES: external ressitance
* Return Value : None
***********************************************************************************************************************/
void LCD_VoltageMode(uint8_t VoltMode)
{
		LCD->LCDM0 &= ~LCD_LCDM0_MDSET_Msk;
		LCD->LCDM0 |= (VoltMode << LCD_LCDM0_MDSET_Pos);     /* set voltage method*/
	
}

/***********************************************************************************************************************
* Function Name: LCD_WaveSelect
* Description  : This function select wave.
* Arguments    : AreaAB
*                  LCD_Wave_A: A wave
*									 LCD_Wave_B: B wave
* Return Value : None
***********************************************************************************************************************/
void LCD_WaveSelect(uint8_t WaveMode)
{
	if(WaveMode == LCD_Wave_A)
	{
		 LCD->LCDM0 &= ~LCD_LCDM0_LWAVE_Msk;    /* select A wave */
	}
	else
	{
		LCD->LCDM0 |= (WaveMode << LCD_LCDM0_LWAVE_Pos);     /* select B wave */
	}
}

/***********************************************************************************************************************
* Function Name: LCD_TimeSliceSelect
* Description  : This function select LCD time slice.
* Arguments    : AreaAB
*                  LCD_TimeSlice_none: stop
*									 LCD_TimeSlice2: 2 time slice
*									 LCD_TimeSlice3: 3 time slice
*									 LCD_TimeSlice4: 4 time slice
*									 LCD_TimeSlice6: 6 time slice
*									 LCD_TimeSlice8: 8 time slice
* Return Value : None
***********************************************************************************************************************/
void LCD_TimeSliceSelect(uint8_t TimeSlice)
{
	uint8_t Temp;
	
	if(TimeSlice == LCD_TimeSlice_none)
	{
		 LCD->LCDM0 &= ~LCD_LCDM0_LDTY_Msk;    /* select stop time slice */
	}
	else
	{
		Temp =  LCD->LCDM0;
		Temp &= ~LCD_LCDM0_LDTY_Msk; 
		Temp |= (TimeSlice << LCD_LCDM0_LDTY_Pos); 
		LCD->LCDM0 = Temp;     								/* set time slice */
	}
}

/***********************************************************************************************************************
* Function Name: LCD_BiasSelect
* Description  : This function select LCD Bias.
* Arguments    : BiasVal
*                  LCD_Bias_Half: 1/2 bias
*									 LCD_Bias_Third: 1/3 bias
*									 LCD_Bias_Quarter: 1/4 bias
* Return Value : None
***********************************************************************************************************************/
void LCD_BiasSelect(uint8_t BiasVal)
{
	uint8_t Temp;
	
	Temp =  LCD->LCDM0;
	Temp &= ~LCD_LCDM0_LBAS_Msk; 
	Temp |= (BiasVal << LCD_LCDM0_LBAS_Pos); 
	
	LCD->LCDM0 = Temp;     								/* set bias */
}

	
/***********************************************************************************************************************
* Function Name: LCD_DispAreaSelect
* Description  : This function select display area A or B.
* Arguments    : AreaAB
*                  LCD_AreaA_Select: Graphic A
*									 LCD_AreaB_Select: Graphic B
* Return Value : None
***********************************************************************************************************************/
void LCD_DispAreaSelect(uint8_t AreaAB)
{
	if(AreaAB == LCD_AreaA_Select)
	{
		 LCD->LCDM1 &= ~LCD_LCDM1_LCDSEL_Msk;    /* select A Graphics area */
	}
	else
	{
		LCD->LCDM1 |= LCD_LCDM1_LCDSEL_Msk;     /* select B Graphics area */
	}
}

/***********************************************************************************************************************
* Function Name: LCD_InitVoltageSelect
* Description  : This function select the LCD initial voltage.
* Arguments    : InitV
*                  LCD_Init_Voltage2700MV: 2.7V
*									 LCD_Init_Voltage4200MV: 4.2V
* Return Value : None
***********************************************************************************************************************/
void LCD_InitVoltageSelect(uint8_t InitV)
{
	if(InitV == LCD_Init_Voltage2700MV)
	{
		 LCD->LCDM1 &= ~LCD_LCDM1_LCDVLM_Msk;    /* select A Graphics area */
	}
	else
	{
		LCD->LCDM1 |= LCD_LCDM1_LCDVLM_Msk;     /* select B Graphics area */
	}
}

/***********************************************************************************************************************
* Function Name: LCD_CLK_Select
* Description  : This function select LCD clock.
* Arguments    : LCDCLK
*                  LCD_DIV_fSUBfIL5: fSUB2^5 or fIL2^5
*                  LCD_DIV_fSUBfIL6: fSUB2^6 or fIL2^6
*                  LCD_DIV_fSUBfIL7: fSUB2^7 or fIL2^7
*                  LCD_DIV_fSUBfIL8: fSUB2^8 or fIL2^8
*                  LCD_DIV_fSUBfIL9: fSUB2^9 or fIL2^9
*                  LCD_DIV_fSUB10:   fSUB2^10
*                  LCD_DIV_fMAIN10:  fMAIN2^10
*                  LCD_DIV_fMAIN11:  fMAIN2^11
*                  LCD_DIV_fMAIN12:  fMAIN2^12
*                  LCD_DIV_fMAIN13:  fMAIN2^13
*                  LCD_DIV_fMAIN14:  fMAIN2^14
*                  LCD_DIV_fMAIN15:  fMAIN2^15
*                  LCD_DIV_fMAIN16:  fMAIN2^16
*                  LCD_DIV_fMAIN17:  fMAIN2^17
*                  LCD_DIV_fMAIN18:  fMAIN2^18
*                  LCD_DIV_fMAIN19:  fMAIN2^19
* Return Value : None
***********************************************************************************************************************/
void LCD_CLK_Select(uint8_t LCDCLK)
{
	LCD->LCDC0 = LCDCLK;
}

/***********************************************************************************************************************
* Function Name: LCD_Voltage_RiseLevel
* Description  : This function select LCD base voltage level.
* Arguments    : LCDCLK
* 									LCD_Voltage_1V		  :base voltage is 1.00V
* 									LCD_Voltage_1050MV	:base voltage is 1.05V
* 									LCD_Voltage_1100V		:base voltage is 1.10V
* 									LCD_Voltage_1150V		:base voltage is 1.15V
* 									LCD_Voltage_1200V		:base voltage is 1.20V
* 									LCD_Voltage_1250V  	:base voltage is 1.25V
* 									LCD_Voltage_1300V  	:base voltage is 1.30V
* 									LCD_Voltage_1350V  	:base voltage is 1.35V
* 									LCD_Voltage_1400V  	:base voltage is 1.40V
* 									LCD_Voltage_1450V  	:base voltage is 1.45V
* 									LCD_Voltage_1500V  	:base voltage is 1.50V
* 									LCD_Voltage_1550V  	:base voltage is 1.55V
* 									LCD_Voltage_1600V  	:base voltage is 1.60V
* 									LCD_Voltage_1650V  	:base voltage is 1.65V
* 									LCD_Voltage_1700V  	:base voltage is 1.70V
* 									LCD_Voltage_1750V  	:base voltage is 1.75V
* Return Value : None
***********************************************************************************************************************/
void LCD_Voltage_RiseLevel(uint8_t LCDVLT)
{
	LCD->VLCD = LCDVLT;
}

/***********************************************************************************************************************
* Function Name: LCD_SmitContrl
* Description  : This function enables the IO SMIT function.
* Arguments    : SMITVLx
* 									LCD_SMIT_VLxOFF  	:SMIT on VLx no work
* 									LCD_SMIT_VLxON  	:SMIT on VLx Open
* Arguments    : SMITCAP
* 									LCD_SMIT_CAPLHOFF  	:SMIT on CAPL/CAPH no work
* 									LCD_SMIT_CAPLHON  	:SMIT on CAPL/CAPH Open
* Return Value : None
***********************************************************************************************************************/
void LCD_SmitContrl(uint8_t SMITVLx, uint8_t SMITCAP)
{
  PPORGA->ISCLCD = SMITVLx | SMITCAP;    /* Enable LCD clock operation */
}

/***********************************************************************************************************************
* Function Name: R_LCD_Start
* Description  : This function enables the LCD display.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LCD_Start(void)
{
    LCD->LCDM1 |= LCD_LCDM1_LCDON_Msk;    /* Enable LCD clock operation */
}
/***********************************************************************************************************************
* Function Name: R_LCD_Stop
* Description  : This function disables the LCD display.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LCD_Stop(void)
{
    LCD->LCDM1 &= ~LCD_LCDM1_LCDON_Msk;    /* disable LCD clock operation */
}
/***********************************************************************************************************************
* Function Name: R_LCD_Voltage_On
* Description  : This function enables voltage boost circuit or capacitor split circuit.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LCD_Voltage_On(void)
{
    volatile uint32_t wt_count;

    LCD->LCDM1 |= LCD_LCDM1_VLCON_Msk;

    /* Change the waiting time according to the system */
    for(wt_count = 0U; wt_count <= LCD_VOLTAGE_WAITTIME; wt_count++)
    {
        __NOP();
    }

    LCD->LCDM1 |= LCD_LCDM1_SCOC_Msk;
}
/***********************************************************************************************************************
* Function Name: R_LCD_Voltage_Off
* Description  : This function disables voltage boost circuit or capacitor split circuit.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LCD_Voltage_Off(void)
{
    LCD->LCDM1 &= ~(LCD_LCDM1_SCOC_Msk + LCD_LCDM1_VLCON_Msk);
    LCD->LCDM0 |= (uint8_t)_C0_LCD_VOLTAGE_MODE_EXTERNAL;
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
