/*******************************************************************************
* Copyright (C) 2019 China Micro Semiconductor Limited Company. All Rights Reserved.
*
* This software is owned and published by:
* CMS LLC, No 2609-10, Taurus Plaza, TaoyuanRoad, NanshanDistrict, Shenzhen, China.
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with CMS
* components. This software is licensed by CMS to be adapted only
* for use in systems utilizing CMS components. CMS shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein. CMS is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/****************************************************************************/
/** \file opa.c
**
**	History:
**	
*****************************************************************************/
/****************************************************************************/
/*	include files
*****************************************************************************/
#include "opa.h"
/****************************************************************************/
/*	Local pre-processor symbols/macros('#define')
*****************************************************************************/

/****************************************************************************/
/*	Global variable definitions(declared in header file with 'extern')
*****************************************************************************/

/****************************************************************************/
/*	Local type definitions('typedef')
*****************************************************************************/

/****************************************************************************/
/*	Local variable  definitions('static')
*****************************************************************************/


/****************************************************************************/
/*	Local function prototypes('static')
*****************************************************************************/

/****************************************************************************/
/*	Function implementation - global ('extern') and local('static')
*****************************************************************************/

/*****************************************************************************
 ** \brief	 OPA_ConfigRunMode
 **			 ˷ģʽ
 ** \param [in] 
 **				mode: (1)OPA_Mode_StableCurrent
 **					  (2)OPA_Mode_Buff
 ** \return  none
 ** \note    
 *****************************************************************************/
void OPA_ConfigRunMode(uint32_t mode)
{
	OPA->OPACTL &= ~(OPA_OPACTL_OPAMODE_Msk);
	OPA->OPACTL |= mode << OPA_OPACTL_OPAMODE_Pos;		
}

/*****************************************************************************
 ** \brief	 OPA_ConfigPositive
 **			 ˷
 ** \param [in] 
 **				PositiveSource:  (1)OPA_Input_5bitDAC
 **								 (2)OPA_Input_8bitDAC
 ** \return  none
 ** \note    
 *****************************************************************************/
void OPA_ConfigPositive(uint32_t PositiveSource)
{
	OPA->OPACTL &= ~(OPA_OPACTL_INPSEL_Msk);
	OPA->OPACTL |= PositiveSource << OPA_OPACTL_INPSEL_Pos;
}

/*****************************************************************************
 ** \brief	 OPA_ConfigCurrent
 **			 ˷
 ** \param [in] 
 **				CurrentSource:  (1)OPA_Current_none
 **								 (2)OPA_Current_on
 ** \return  none
 ** \note    
 *****************************************************************************/
void OPA_ConfigCurrent(uint32_t CurrentSource)
{
	OPA->OPACTL &= ~(OPA_OPACTL_RPDMODE_Msk);
	OPA->OPACTL |= CurrentSource << OPA_OPACTL_RPDMODE_Pos;
}

/*****************************************************************************
 ** \brief	 OPA_ConfigDACPower
 **			 ˷
 ** \param [in] 
 **				DACPower:  (1)OPA_5bitDacP_OpaP
 **								 (2)OPA_5bitDacP_Ext
 ** \return  none
 ** \note    
 *****************************************************************************/
void OPA_ConfigDACPower(uint32_t DACPower)
{
	OPA->OPADAC &= ~(OPA_OPADAC_OPADAP_Msk);
	OPA->OPADAC |= DACPower << OPA_OPADAC_OPADAP_Pos;
}

/*****************************************************************************
 ** \brief	 OPA_SetOut
 **			 5bit DAC
 ** \param [in] n:0~31
 ** \return  none
 ** \note    Vout = (19+3*n)/560 *Vref
 *****************************************************************************/
void OPA_SetOut(uint32_t DACOut)
{
	OPA->OPADAC &= ~(OPA_OPADAC_OPADAO_Msk);
	OPA->OPADAC |= DACOut << OPA_OPADAC_OPADAO_Pos;
}

/********************************************************************************
 ** \brief	 OPA_Start
 **			 OP
 ** \param [in] opa :OPA0 OPA1
 ** \return  none
 ** \note   
 ******************************************************************************/
void OPA_Start(void)
{
	OPA->OPACTL |= OPA_OPACTL_OPAEN_Msk;
}

/********************************************************************************
 ** \brief	 OPA_Stop
 **			 رOP
 ** \param [in] opa :OPA0 OPA1
 ** \return  none
 ** \note   
 ******************************************************************************/
void OPA_Stop(void)
{
	OPA->OPACTL &= ~OPA_OPACTL_OPAEN_Msk;
}

void OPA_Init(OPA_InitTypeDef* OPA_InitStruct)
{

	/* Clock gate control register setting to enable SCI0 clock */
	CGC_PER1PeriphClockCmd(CGC_PER1Periph_OPA, ENABLE);
	
	OPA->OPACTL = (OPA_InitStruct->OPA_Current << OPA_OPACTL_RPDMODE_Pos) | (OPA_InitStruct->OPA_Input << OPA_OPACTL_INPSEL_Pos)
              	| (OPA_InitStruct->OPA_Mode << OPA_OPACTL_OPAMODE_Pos);
	
	OPA->OPADAC = (OPA_InitStruct->OPA_5bitDacP << OPA_OPADAC_OPADAP_Pos) | OPA_InitStruct->Dac_Out;
	
	//start opa
	OPA->OPACTL |= OPA_OPACTL_OPAEN_Msk;
}






