/*******************************************************************************
* Copyright (C) 2019 China Micro Semiconductor Limited Company. All Rights Reserved.
*
* This software is owned and published by:
* CMS LLC, No 2609-10, Taurus Plaza, TaoyuanRoad, NanshanDistrict, Shenzhen, China.
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with CMS
* components. This software is licensed by CMS to be adapted only
* for use in systems utilizing CMS components. CMS shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein. CMS is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/****************************************************************************/
/** \file sigmaDeltaAdc.c
**
**  
**
**	History:
**	
*****************************************************************************/
/****************************************************************************/
/*	include files
*****************************************************************************/
#include "sigmadeltaadc.h"
/****************************************************************************/
/*	Local pre-processor symbols/macros('#define')
****************************************************************************/

/****************************************************************************/
/*	Global variable definitions(declared in header file with 'extern')
****************************************************************************/
SIGMA_DELTA_ADC_INFO_S sigmaDeltaAdcInfo={0};

REG_CON1_U regCfg1Info={0};
REG_CON2_U regCfg2Info={0};
REG_CON3_U regCfg3Info={0};
REG_CON4_U regCfg4Info={0};
/****************************************************************************/
/*	Local type definitions('typedef')
****************************************************************************/

/****************************************************************************/
/*	Local variable  definitions('static')
****************************************************************************/

/****************************************************************************/
/*	Local function prototypes('static')
****************************************************************************/

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_Delay	
						 ʱ޸delay_cntԵSigma Delta Adcͨ 
 ** \param [in] none       
 ** \return  none 
*****************************************************************************/
void SigmaDeltaAdc_Delay(void)
{
	uint8_t delay_cnt=3;
	while(delay_cnt--)
	{
	    ;
	}
	//PORT_ToggleBit(PORTD,PIN2);
}


/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_OneClock
						 һSCLK
 ** \param [in] none       
 ** \return  none
*****************************************************************************/
void SigmaDeltaAdc_OneClock(void)
{   

    SIGMA_DELTA_ADC_SCK_H;   
    SigmaDeltaAdc_Delay();
	SIGMA_DELTA_ADC_SCK_L;
    SigmaDeltaAdc_Delay();

}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_WriteOneBit
 **			  		дһbit
 ** \param [in] 
 **					 sda_dat: 01
 ** \return  none
*****************************************************************************/
void SigmaDeltaAdc_WriteOneBit(uint8_t sda_dat)
{

    if(sda_dat)
        W_SIGMA_DELTA_ADC_SDA_H;
    else
        W_SIGMA_DELTA_ADC_SDA_L;
    
    SIGMA_DELTA_ADC_SCK_H;   
    SigmaDeltaAdc_Delay();
	SIGMA_DELTA_ADC_SCK_L;
    SigmaDeltaAdc_Delay();

}
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_ReadOneBit
 **			  	 һbit
 ** \param [in] none
 ** \return  10
*****************************************************************************/
uint8_t SigmaDeltaAdc_ReadOneBit(void)
{

    uint8_t read_dat=0;
    
    SIGMA_DELTA_ADC_SCK_H;   
    SigmaDeltaAdc_Delay();
	SIGMA_DELTA_ADC_SCK_L;

    if(R_SIGMA_DELTA_ADC_SDA_H) read_dat = 1;
    
    SigmaDeltaAdc_Delay();
    return read_dat;

}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_WriteReg
 **			 		 дSigma Delta Adc
 ** \param [in] wr_cmd: 
								CFG1_CMD_WRITE  
								CFG2_CMD_WRITE
								CFG3_CMD_WRITE
								CFG4_CMD_WRITE 0x61
 **							datֵ
 ** \return  none
 ** \note 	 
*****************************************************************************/
void SigmaDeltaAdc_WriteReg(uint8_t wr_cmd,uint8_t dat)
{
	uint8_t i=0,temp=0;
		
    
	for(i=0;i<24;i++)//1~24  
		SigmaDeltaAdc_OneClock();
    

	SigmaDeltaAdc_OneClock();//25		
	SigmaDeltaAdc_OneClock();//26		
	SigmaDeltaAdc_WriteOneBit(1);//27,not must,it's same as adc logic


    
	SigmaDeltaAdc_OneClock();//28
	SigmaDeltaAdc_OneClock();//29  
  	ADC_SDA_OUTPUT_MODE;


	wr_cmd = wr_cmd<<1;

	for(i=0;i<7;i++)//30~36
	{
    temp = wr_cmd << i;
		if(temp&0x80)
			SigmaDeltaAdc_WriteOneBit(1);
		else
			SigmaDeltaAdc_WriteOneBit(0);
		
	}	  
	SigmaDeltaAdc_OneClock();//37


	for(i=0;i<8;i++)//38~45
	{
    temp = dat << i;
		if(temp&0x80)
			SigmaDeltaAdc_WriteOneBit(1);
		else
			SigmaDeltaAdc_WriteOneBit(0);
		
	}
    
	ADC_SDA_INPUT_MODE;	
	SigmaDeltaAdc_OneClock();//46
    
}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_ReadReg
 **			 		 Sigma Delta Adc
 ** \param [in] wr_cmd: 
								CFG1_CMD_READ  
								CFG2_CMD_READ
								CFG3_CMD_READ
								CFG4_CMD_READ
 ** \return  ֵ
 ** \note 	 
*****************************************************************************/
uint8_t SigmaDeltaAdc_ReadReg(uint8_t rd_cmd)
{
	uint8_t i=0,temp=0,rec_dat=0;
		
    
	for(i=0;i<24;i++)//1~24  
		SigmaDeltaAdc_OneClock();
    

	SigmaDeltaAdc_OneClock();//25		
	SigmaDeltaAdc_OneClock();//26		
	SigmaDeltaAdc_WriteOneBit(1);//27,not must,it's same as adc logic


    
	SigmaDeltaAdc_OneClock();//28
	SigmaDeltaAdc_OneClock();//29  
    ADC_SDA_OUTPUT_MODE;


	rd_cmd = rd_cmd<<1;
	for(i=0;i<7;i++)//30~36
	{
    temp = rd_cmd << i;
		if(temp&0x80)
			SigmaDeltaAdc_WriteOneBit(1);
		else
			SigmaDeltaAdc_WriteOneBit(0);
		
	}	
    
	SigmaDeltaAdc_OneClock();//37

    ADC_SDA_INPUT_MODE;	
    
	for(i=0;i<8;i++)//38~45
	{
		rec_dat <<= 1;
		if(SigmaDeltaAdc_ReadOneBit())  rec_dat++;
		else ;
	}

    
	ADC_SDA_INPUT_MODE;	
	SigmaDeltaAdc_OneClock();//46
	return rec_dat; 
}
