/*******************************************************************************
* Copyright (C) 2019 China Micro Semiconductor Limited Company. All Rights Reserved.
*
* This software is owned and published by:
* CMS LLC, No 2609-10, Taurus Plaza, TaoyuanRoad, NanshanDistrict, Shenzhen, China.
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with CMS
* components. This software is licensed by CMS to be adapted only
* for use in systems utilizing CMS components. CMS shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein. CMS is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/****************************************************************************/
/** \file demo_sigmaDeltaAdc.c
**
**  
**
**	History:
**	
*****************************************************************************/
/****************************************************************************/
/*	include files
*****************************************************************************/
#include "sigmadeltaadc_demo.h"
#include "intp_demo.h"
/****************************************************************************/
/*	Local pre-processor symbols('#define')
****************************************************************************/

/****************************************************************************/
/*	Global variable definitions(declared in header file with 'extern')
****************************************************************************/

/****************************************************************************/
/*	Local type definitions('typedef')
****************************************************************************/
INTP_InitTypeDef intp4_initTypeDef = {INTP4, Trigger_Falling};
/****************************************************************************/
/*	Local variable  definitions('static')
****************************************************************************/

/****************************************************************************/
/*	Local function prototypes('static')
****************************************************************************/

/****************************************************************************/
/*	Function implementation - global ('extern') and local('static')
****************************************************************************/
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_OutPutRate
 **			 		 Sigma Delta Adcת
 ** \param [in] speed: ODR_Eöֵ
								adc_fre  ADC_FRE_Eöֵ

 ** \return  none
 ** \note 	 
*****************************************************************************/
void SigmaDeltaAdc_OutPutRate(ODR_E speed,ADC_FRE_E adc_fre)
{
	regCfg1Info.bitCfg.osr = (speed >> 1) &0x03;
	regCfg2Info.bitCfg.osr = speed & 0x01;

	regCfg2Info.bitCfg.fadc = adc_fre;
}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_PgaSet
 **			 		 Sigma Delta Adc PGAŴ
 ** \param [in] pga: PGA_Eöֵ
 ** \return  none
 ** \note 	 
*****************************************************************************/
void SigmaDeltaAdc_PgaSet(PGA_E pga)
{
	regCfg1Info.bitCfg.pgaSel = pga & 0x07;
}
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_LdoSet
 **			 		 Sigma Delta Adc LDOѹ
 ** \param [in] ldo_vol: LDO_Eöֵ
 ** \return  none
 ** \note 	 
*****************************************************************************/
void  SigmaDeltaAdc_LdoSet(LDO_E ldo_vol)
{
	regCfg1Info.bitCfg.setLdo =  ldo_vol & 0x03; 
	regCfg3Info.bitCfg.bypassLdo = ((ldo_vol >> 2) & 0x01);
}
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_ChanSel
 **			 		 Sigma Delta Adc LDOͨͨ
 ** \param [in] chan_sel: CHAN_SEL_Eöֵ
 **							chan_typeCHAN_TYPE_Eöֵ
 ** \return  none
 ** \note 	 
*****************************************************************************/
void  SigmaDeltaAdc_ChanSel(CHAN_SEL_E chan_sel,CHAN_TYPE_E chan_type)
{
	regCfg1Info.bitCfg.chSel = chan_sel;
	regCfg2Info.bitCfg.chSel = chan_type;
}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_ReadyInit	
						 Sigma Delta AdcԤʼIOڡ˿жϺͼĴʼֵ
 ** \param [in] none       
 ** \return  none
*****************************************************************************/
void SigmaDeltaAdc_ReadyInit(void)
{	
	GPIO_Output_Enable(&PORT->PC,GPIO_Pin_13);	//CLK
	GPIO_Output_Enable(&PORT->PC,GPIO_Pin_14);	//SDO

	INTP4_PORT_SETTING();
	INTP_Init(&intp4_initTypeDef);	
	ISR_Register(INTP4_IRQn, intp_interrupt); //жϷע
	
	SIGMA_DELTA_ADC_SCK_H;
	regCfg1Info.byteCfg = REG1_DEFAULT_CFG;
	regCfg2Info.byteCfg = REG2_DEFAULT_CFG;
	regCfg3Info.byteCfg = REG3_DEFAULT_CFG;
	regCfg4Info.byteCfg = REG4_DEFAULT_CFG;

	sigmaDeltaAdcInfo.phase = PHASE_READ_DAT;
	
	INTP_Start(INTP4);
}

/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_Init	
						 Sigma Delta AdcʼжЩĴҪд
 ** \param [in] none       
 ** \return  none
*****************************************************************************/
void  SigmaDeltaAdc_Init(void)
{
	static uint8_t reg1byte=REG1_DEFAULT_CFG,reg2byte=REG2_DEFAULT_CFG,reg3byte=REG3_DEFAULT_CFG,reg4byte=REG4_DEFAULT_CFG;
	
	
	sigmaDeltaAdcInfo.isCfg1Update=0;
	sigmaDeltaAdcInfo.isCfg2Update=0;
	sigmaDeltaAdcInfo.isCfg3Update=0;
	sigmaDeltaAdcInfo.isCfg4Update=0;
	
	SigmaDeltaAdc_LdoSet(SET_LDO_2P4V);
	SigmaDeltaAdc_ChanSel(CH_SEL_CH1,CH_TYPE_CHOP);
	SigmaDeltaAdc_PgaSet(PGA_64);
	SigmaDeltaAdc_OutPutRate(ODR_80HZ_ADCFRE_328KHZ,ADCFRE_656KHZ);


	if(reg1byte != regCfg1Info.byteCfg)
	{
		sigmaDeltaAdcInfo.isCfg1Update=1;
	}
	
	if(reg2byte != regCfg2Info.byteCfg)	
	{
		sigmaDeltaAdcInfo.isCfg2Update=1;
	}
	
	if(reg3byte != regCfg3Info.byteCfg)
	{
		sigmaDeltaAdcInfo.isCfg3Update=1;
	}
	
	if(reg4byte != regCfg4Info.byteCfg)
	{
		sigmaDeltaAdcInfo.isCfg4Update=1;
	}

	reg1byte = regCfg1Info.byteCfg;
	reg2byte = regCfg2Info.byteCfg;
	reg3byte = regCfg3Info.byteCfg;
	reg4byte = regCfg4Info.byteCfg;

	printf("CFG:cfg1=%x,cfg2=%x,cfg3=%x,cfg4=%x\n",regCfg1Info.byteCfg,regCfg2Info.byteCfg,regCfg3Info.byteCfg,regCfg4Info.byteCfg);
	
	SIGMA_DELTA_ADC_SCK_L;
	sigmaDeltaAdcInfo.phase = PHASE_WRITE_CFG;
	
}
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_ReadVal	
						 ȡSigma Delta Adcֵ
 ** \param [in] adc_info SIGMA_DELTA_ADC_INFO_Sṹ      
 ** \return  Sigma Delta Adcֵ
*****************************************************************************/
int32_t SigmaDeltaAdc_ReadVal(SIGMA_DELTA_ADC_INFO_S *adc_info)
{
	uint8_t i=0,is_val_negative=FALSE;
  	uint32_t temp;

	sigmaDeltaAdcInfo.adcConvertCnt++;
	
	for(i=0;i<24;i++)//1~24
	{
        temp = temp << 1;
		if(SigmaDeltaAdc_ReadOneBit())	temp++;
		else ;
		
	}
	
  	temp = temp & 0xFFFFFF;
	if(temp & 0x800000)	is_val_negative = TRUE;
	adc_info->adcVal = temp & 0xFFFFFF;


	if(TRUE == is_val_negative)
	{
		adc_info->adcVal = 0x1000000-adc_info->adcVal;
	    adc_info->adcVal = 0 - adc_info->adcVal;
	}
	
	adc_info->sampleOk = TRUE;
	//adc_info->adcVal >>= 4;
	return adc_info->adcVal;
}
/*****************************************************************************
 ** \brief	 SigmaDeltaAdc_Handle	
						 Sigma Delta AdcֱдúͶȡadcֵ
 ** \param 	 none      
 ** \return  none
*****************************************************************************/
void SigmaDeltaAdc_Handle(void)
{
	uint8_t read_cfg;
	switch (sigmaDeltaAdcInfo.phase)
	{
		case PHASE_WRITE_CFG:
		{
			if(sigmaDeltaAdcInfo.isCfg1Update)
			{
				SigmaDeltaAdc_WriteReg(CFG1_CMD_WRITE,regCfg1Info.byteCfg);
				sigmaDeltaAdcInfo.isCfg1Update = FALSE;
			}
			else if(sigmaDeltaAdcInfo.isCfg2Update)
			{
				SigmaDeltaAdc_WriteReg(CFG2_CMD_WRITE,regCfg2Info.byteCfg);
				sigmaDeltaAdcInfo.isCfg2Update = FALSE;
			}
			else if(sigmaDeltaAdcInfo.isCfg3Update)	
			{
				SigmaDeltaAdc_WriteReg(CFG3_CMD_WRITE,regCfg3Info.byteCfg);
				sigmaDeltaAdcInfo.isCfg3Update = FALSE;
			}
			else if(sigmaDeltaAdcInfo.isCfg4Update)	
			{
				SigmaDeltaAdc_WriteReg(CFG4_CMD_WRITE,regCfg4Info.byteCfg);
				sigmaDeltaAdcInfo.isCfg4Update = FALSE;
			}
			else
			{
				sigmaDeltaAdcInfo.phase = PHASE_READ_CFG;
				sigmaDeltaAdcInfo.readCfgCnt = 0;
			}			
		}
		break;
		case PHASE_READ_CFG:
		{
			if(0 == sigmaDeltaAdcInfo.readCfgCnt)
			{
				read_cfg = SigmaDeltaAdc_ReadReg(CFG1_CMD_READ);
				sigmaDeltaAdcInfo.readCfgCnt++;
			}
			else if(1 == sigmaDeltaAdcInfo.readCfgCnt)
			{
				read_cfg = SigmaDeltaAdc_ReadReg(CFG2_CMD_READ);
				sigmaDeltaAdcInfo.readCfgCnt++;
			}
			else if(2 == sigmaDeltaAdcInfo.readCfgCnt)
			{
				read_cfg = SigmaDeltaAdc_ReadReg(CFG3_CMD_READ);
				sigmaDeltaAdcInfo.readCfgCnt++;
			}
			else if(3 == sigmaDeltaAdcInfo.readCfgCnt)
			{
				read_cfg = SigmaDeltaAdc_ReadReg(CFG4_CMD_READ);
				sigmaDeltaAdcInfo.readCfgCnt++;
			}
			printf("Read%d=%x\n",sigmaDeltaAdcInfo.readCfgCnt,read_cfg);
			if(4 == sigmaDeltaAdcInfo.readCfgCnt)
			{
				sigmaDeltaAdcInfo.phase = PHASE_READ_DAT;
			}
		}
		break;
		case PHASE_READ_DAT:
			SigmaDeltaAdc_ReadVal(&sigmaDeltaAdcInfo);

		break;		
	}

}


/******************************************************************************
* Function Name: Sigmadeltaadc_Init
* @brief  init Sigma_deltaadc demo
* @param  none
* @return none
*******************************************************************************/
void Sigmadeltaadc_Init(void)
{
	int8_t ret;
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	
	sigmaDeltaAdcInfo.isCfg1Update=0;				//initial update status
	sigmaDeltaAdcInfo.isCfg2Update=0;
	sigmaDeltaAdcInfo.isCfg3Update=0;
	sigmaDeltaAdcInfo.isCfg4Update=0;
	
/*******config IO CLK********************************/		
	GPIO_PinAFConfig(GPIO_PORTC, GPIO_Pin_13, GROUP_AF_ODEFAULT);   	//CLK
	
	GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_13;
	GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Ctrl  = GPIO_Control_DIG;
	GPIO_Init(GPIO_PORTC, &GPIO_InitStruct);

/*******read register********************************/	
	SIGMA_DELTA_ADC_SCK_H;
	regCfg1Info.byteCfg = REG1_DEFAULT_CFG;
	regCfg2Info.byteCfg = REG2_DEFAULT_CFG;
	regCfg3Info.byteCfg = REG3_DEFAULT_CFG;
	regCfg4Info.byteCfg = REG4_DEFAULT_CFG;
	sigmaDeltaAdcInfo.phase = PHASE_READ_DAT;
	INTP_Start(INTP4);
	
	ret=255;				//waitting read 
	while(ret--);
/*******config register********************************/			
	SigmaDeltaAdc_LdoSet(SET_LDO_2P4V);
	SigmaDeltaAdc_ChanSel(CH_SEL_CH1,CH_TYPE_CHOP);
	SigmaDeltaAdc_PgaSet(PGA_64);
	SigmaDeltaAdc_OutPutRate(ODR_80HZ_ADCFRE_328KHZ,ADCFRE_656KHZ);


	if(REG1_DEFAULT_CFG != regCfg1Info.byteCfg)					//judge the data modified or not
	{
		sigmaDeltaAdcInfo.isCfg1Update=1;
	}
	if(REG2_DEFAULT_CFG != regCfg2Info.byteCfg)	
	{
		sigmaDeltaAdcInfo.isCfg2Update=1;
	}
	if(REG3_DEFAULT_CFG != regCfg3Info.byteCfg)
	{
		sigmaDeltaAdcInfo.isCfg3Update=1;
	}
	if(REG4_DEFAULT_CFG != regCfg4Info.byteCfg)
	{
		sigmaDeltaAdcInfo.isCfg4Update=1;
	}
//print write register
	printf("CFG:cfg1=%x,cfg2=%x,cfg3=%x,cfg4=%x\n",regCfg1Info.byteCfg,regCfg2Info.byteCfg,regCfg3Info.byteCfg,regCfg4Info.byteCfg);
	
	SIGMA_DELTA_ADC_SCK_L;
	sigmaDeltaAdcInfo.phase = PHASE_WRITE_CFG;       //write register

}






